#ifndef CONFIG_H_
#define CONFIG_H_

#include "drivers.h"

extern int GP32_NEW_BLU;


#ifdef __WIN32__
	#define ROMS_DIR "c:\\GP32\\MAME150\\GPMM\\mamegp32\\roms\\"
	#define CFG_DIR "c:\\GP32\\MAME150\\GPMM\\mamegp32\\CFG\\"
	#define IMG_DIR "c:\\GP32\\MAME150\\GPMM\\mamegp32\\IMAGES\\"

  #define BUTTON_ESCAPE SDLK_ESCAPE
  #define BUTTON_DOWN SDLK_DOWN
  #define BUTTON_UP SDLK_UP
  #define BUTTON_LEFT SDLK_LEFT
  #define BUTTON_RIGHT SDLK_RIGHT
  #define BUTTON_LS SDLK_q
  #define BUTTON_RS SDLK_w
  #define BUTTON_A SDLK_a
  #define BUTTON_B SDLK_s
  #define BUTTON_START SDLK_z
  #define BUTTON_SELECT SDLK_x
#else
	#define ROMS_DIR "/gpmm/mamegp32/roms/"
	#define CFG_DIR "/gpmm/mamegp32/cfg/"
	#define IMG_DIR "/gpmm/mamegp32/images/"

  #define BUTTON_ESCAPE SDLK_ESCAPE
  #define BUTTON_DOWN SDLK_DOWN
  #define BUTTON_UP SDLK_UP
  #define BUTTON_LEFT SDLK_LEFT
  #define BUTTON_RIGHT SDLK_RIGHT
  #define BUTTON_LS 8
  #define BUTTON_RS 9
  #define BUTTON_A SDLK_LCTRL
  #define BUTTON_B SDLK_LALT
  #define BUTTON_START 13
  #define BUTTON_SELECT 32
#endif

typedef enum{
	OP_EQUALS,
	OP_STARTSWITH,
	OP_CONTAINS,
	OP_REGEXP
} OP_CODES;


typedef enum {
  FIELD_ALL,
  FIELD_GAME,
  FIELD_DESC,
  FIELD_MANUFACTURER,
  FIELD_FXE,
  FIELD_YEAR,
  FIELD_CLONEOF,
  FIELD_GENRE,
  FIELD_CATEGORY
} FIELD_CODES;



typedef struct MISSING_DRIVER
{
	char name[12];
	char crlf[2];
} MISSING_DRIVER;

typedef struct DRIVERLOC
{
	char Desc[31];
	char Path[81];
} DRIVERLOC;


typedef struct FILTER
{
	char FilterDesc[25];
	int Field;
	int Op;
	void **Values;
	
	int SelectedGameFilterID; /* position of current selected game (filter pos) */
	int GamesMatchingFilter;  /* amount of games that can be displayed with current filter */

	int __SelectedGameID;  /* literal game # */
	int __TotalGames;  /* litteral count of games (alias to NUMGAMES in gp32_frontend_list.h) */
} FILTER;


typedef struct FILTER_GROUP
{
	char FilterGroupDesc[51];
	
	FILTER *Filters;
	int FiltersAllocated;
	int FilterCount;
	int CurrentFilter;
} FILTER_GROUP;



typedef struct CONFIG_OPTIONS
{
  char DefaultFilterGroup[25];
  char DefaultFilter[25];
  
  int RepeatDelay;
  int RepeatRate;
  int UpdateImageFreq;  
  
  int AutoFXEFilter;
  int AutoYearFilter;
  int AutoCloneOfFilter;
  int AutoGenreFilter;
  int AutoCategoryFilter;
  int AutoManufacturerFilter;

  FILTER_GROUP *Groups;
  int GroupCount;
  int GroupsAllocated;
  int CurrentGroup;
  
  DRIVERLOC *DriverLocations;   /* Alternate driver locations (all games) */
  int DriverLocationsCount;
  int DriverLocationsAllocated;
  int CurrentDriverLocation;

  DRIVERLOC *OptDrivers;   /* Optimal driver (game secific) */
  int OptDriverCount;
  int OptDriverAllocated;
  int CurrentOptDriver;  
  int OptDriverDisabled;  /* if true, dont use optimal drivers */
  
  int FilterMissingGames;    
  
} CONFIG_OPTIONS;


void ReadConfigFile(void);
void SetInitialFilterState(void);


extern CONFIG_OPTIONS Config;

#endif /*CONFIG_H_*/
