#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

#include "config.h"
#include "util.h"
#include "filter.h"
#include "video.h"
#include "regexp.h"
#include "missinggames.h"


static MISSING_DRIVER *MissingDrivers=NULL;
static int MissingDriversCount=0;

static int GameExists(char *game);

int IsGameMissing(char *name)
{
  int x;
  for(x = 0; x < MissingDriversCount; x++)
  {
  	if(strcasecmp(MissingDrivers[x].name, name) == 0)
  	  return 1;
  }	
  return 0;
}

void ReadMissing(void)
{
	FILE *fp;
	
	if(MissingDrivers)  free(MissingDrivers);
	MissingDrivers=NULL;
	MissingDriversCount = 0;
		
	fp = fopen(CFG_DIR "missing.cfg", "rb");
	if(!fp) 
	{
		if(Config.FilterMissingGames == 0) return;  /* don't build it if option is turned off */
		
		BuildMissingCfg();
    fp = fopen(CFG_DIR "missing.cfg", "rb");
    if(!fp) 
    {
    	pause("uanble to read or create missing.cfg");
    	return;
    }
	}
	
	fseek(fp, 0, SEEK_END);
	MissingDriversCount = ftell(fp);
	MissingDriversCount = MissingDriversCount / sizeof(MISSING_DRIVER);

  MissingDrivers = (MISSING_DRIVER *) malloc(sizeof(MISSING_DRIVER) * MissingDriversCount);
  if(!MissingDrivers)
  {
  	pause("Unabled to allocate %d for missing.cfg", sizeof(MISSING_DRIVER) * MissingDriversCount);
  	MissingDriversCount = 0;
  	return;
  }
	fseek(fp, 0, SEEK_SET);
	fread(MissingDrivers, sizeof(MISSING_DRIVER), MissingDriversCount, fp);
	fclose(fp);
}


void BuildMissingCfg(void)
{
	MISSING_DRIVER md;
	FILE *fp;
	int x;

	SDL_Rect R1 = RECT(0, 60, 320, 20);
	SDL_Rect R2 = RECT(0, 80, 320, 20);

	strcpy(md.crlf, "\r\n");
	
	SetFontColor(255, 255, 255, 255);
  gfxPrimitivesSetFont(&SDL_gfx_font_7x13B_fnt, 7, 13);
	
	fp = fopen(CFG_DIR "missing.cfg", "wb");
	if(!fp)
	{
		pause("Unable to open missing.cfg for writing");
		return;
	}
	
	for(x=0; x < NUMGAMES; x++)
	{
    SDL_FillRect(screen, &R1, 0); 
		TextAt(100, 60, "[%d/%d] %s", x+1, NUMGAMES, drivers[x].name);
		
		if(!GameExists(drivers[x].name))
		{
			memset(md.name, 0, sizeof(md.name));
			strcpy(md.name, drivers[x].name);
			fwrite(&md, sizeof(MISSING_DRIVER), 1, fp);
			
      SDL_FillRect(screen, &R2, 0); 
			TextAt(100, 80, "%s not found", drivers[x].name);
		}
    SDL_Flip(screen);
	}
	
	fclose(fp);
}

int GameExists(char *game)
{
	char path[81];
	int result;
  struct stat s;

	sprintf(path, "%s%s.zip", ROMS_DIR, game);
/*		result = access(path, &s); */
  result = stat(path, &s);
  if(result == 0) return 1;
  
	sprintf(path, "%s%s", ROMS_DIR, game);
  result = stat(path, &s);
  if(result == 0) return 1;

	return 0;  
}
