#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

#include <SDL_image.h>


#include "config.h"
#include "video.h"
#include "util.h"
#include "filter.h"
#include "selector.h"
#include "menu.h"
#include "StandardHandleEvent.h"
#include "quickfilter.h"


void DrawMameLaunch(void);
static int onKeyDown(SDLKey key, Uint8 *KeyState);
static int onIdle(void);

static SDL_Surface *ICON, *BGI;
static SDL_Rect BGI_R = { 0, 0, 320, 240 };
static Uint32 LastUpdateImage;
static int SelectedGameIDImage;

void UpdateImage(void)
{
	char img[256];
	SDL_Surface *tmp;
	FILTER *Filter = CurrentFilter();
	
	if(!Filter) return;
	
  if(SelectedGameIDImage == Filter->__SelectedGameID ||
     Filter->__SelectedGameID < 0 || 
     Filter->__SelectedGameID >= Filter->__TotalGames)
    return;
	
	if(ICON) SDL_FreeSurface(ICON);
	ICON = NULL;
	SelectedGameIDImage = -1;
	
  sprintf(img, "%s%s.png", IMG_DIR, drivers[Filter->__SelectedGameID].name);
	tmp = IMG_Load(img);
	if(!tmp) return;
	ICON = SDL_DisplayFormat(tmp);
	SDL_SetColorKey(ICON ,SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(screen->format, 0x80, 0x80, 0x80));
	SDL_FreeSurface(tmp);
		
	SelectedGameIDImage = Filter->__SelectedGameID;
	DrawMameLaunch();
}


void MameGameSelector(void)
{
	SDL_Surface *tmp;
  char path[256];

	BGI = NULL;

  sprintf(path, "%sfrontend.png", IMG_DIR);
//sprintf(path, "/GP32/MAME150/GPMM/mamegp32/images/FRONTEND.PNG");
  tmp = IMG_Load(path);
  if(!tmp) pause(SDL_GetError());
	if(tmp)
	{
		BGI = SDL_DisplayFormat(tmp);
		SDL_SetColorKey(BGI ,SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(screen->format, 0x80, 0x80, 0x80));
		SDL_FreeSurface(tmp);
	}
	
	
  DrawMameLaunch();
  StandardHandleEvent(onKeyDown, onIdle);
}

static int onKeyDown(SDLKey key, Uint8 *KeyState)
{
	int x;
	
	/* reset LastUpdateImage so that images don't update while we are scrolling */
	LastUpdateImage = SDL_GetTicks();
	
  switch(key)
  {
    case BUTTON_ESCAPE:
      SDL_Quit();
      exit(0);
      break;

  	case BUTTON_DOWN:
  	  SelectNextGame(); 
    	break;
    	
    case BUTTON_UP:
      SelectPrevGame();
    	break;
    	
    case BUTTON_LEFT:
      for(x=0; x<10; x++)
        SelectPrevGame();
      break;
      
    case BUTTON_RIGHT:
      for(x=0; x<10; x++)
        SelectNextGame();
      break;
      
    case BUTTON_LS:
      if(KeyState[BUTTON_SELECT] || KeyState[BUTTON_START])  /* Select or Start + Left shoulder */
	      GoToPrevFilterGroup(); 
			else		      
        GoToPrevFilter();  /* Left shoulder by itself */
      break;
      
    case BUTTON_RS:
      if(KeyState[BUTTON_SELECT] || KeyState[BUTTON_START])  /* Select or Start + Right shoulder */
	      GoToNextFilterGroup(); 
	    else
        GoToNextFilter();  /* left shoulder by itself */
      break;
      
    case BUTTON_A:
      LaunchDriver();
      break;
      
    case BUTTON_B:
      SelectorMenu();
      break;
      
    case BUTTON_START:
      break;
      
    case BUTTON_SELECT:
	    break;
            
#if 0
    default:
      pause("Key = %d", key);
      break;  
#endif            
  }
  DrawMameLaunch();
  return 0;
}

static int onIdle(void)
{
  if(SDL_GetTicks() - LastUpdateImage > Config.UpdateImageFreq)
  {
  	UpdateImage();
  	LastUpdateImage = SDL_GetTicks();
  }

	return 0;
}


void DrawMameLaunch(void)
{
	int i, screen_x=40, screen_y=38, FirstRow, CurRow=0;
	FILTER *Filter = CurrentFilter();
	
	ClearScreen(); 
	
	if(BGI) SDL_BlitSurface(BGI, NULL, screen, &BGI_R);
	

  FirstRow = Filter->SelectedGameFilterID < 6 ? 0 :  Filter->SelectedGameFilterID - 5;
  if(FirstRow > Filter->GamesMatchingFilter - 11) FirstRow = Filter->GamesMatchingFilter - 11;  
	  
  gfxPrimitivesSetFont(&SDL_gfx_font_9x18B_fnt,9,18);
	SetFontColor(0, 128, 255, 255);
	
#if 0
{
  FILTER_GROUP *fg;
  fg = &Config.Groups[Config.CurrentGroup];
	TextAt(20, 0, "Group [%d/%d] %s", Config.CurrentGroup+1, Config.GroupCount, fg->FilterGroupDesc);
	TextAt(20, 15, "Filter [%d/%d] %s  Pos %d/%d", 
	               fg->CurrentFilter+1, 
	               fg->FilterCount, 
	               fg->Filters[fg->CurrentFilter].FilterDesc,
	               Filter->SelectedGameFilterID + 1,
	               Filter->GamesMatchingFilter);
}                 
#else
	SetFontColor(183, 219, 255, 255);
  gfxPrimitivesSetFont(&SDL_gfx_font_7x13_fnt ,7,13);
  
 	if(QuickFilterStr[0])
	  TextAt(5, 1, "Quick Filter Enabled : %s", CurrentFilter()->FilterDesc);
	else
	  TextAt(5, 1, "Filter: %s", CurrentFilter()->FilterDesc);
  
  TextAt(5, 225, "Group: %s", CurrentFilterGroup()->FilterGroupDesc);

  TextAt(200, 225, "A=Run  B=Menu");
#endif	               


  gfxPrimitivesSetFont(&SDL_gfx_font_9x18B_fnt,9,18);
	SetFontColor(151, 203, 255, 255);

  Filter->__SelectedGameID = -1;
  for (i = 0; i < Filter->__TotalGames; i++)
  {
    if (drivers[i].available)
    {
      if (CurRow >= FirstRow && CurRow <= FirstRow + 10)
      {
        TextAt(screen_x, screen_y, drivers[i].description);
        if (CurRow == Filter->SelectedGameFilterID )
        {
          Filter->__SelectedGameID = i;
        	TextAt(screen_x - 10, screen_y, ">");
        	TextAt(screen_x - 13, screen_y, "-");
        }
        screen_y+=15;
      }
      CurRow++;
    }
  }
  
  if(ICON)
  {
		SDL_Rect ir = RECT((320 - ICON->w) - 20, (240 - ICON->h) - 20, ICON->w, ICON->h); 
		SDL_Rect fr = RECT(ir.x - 2, ir.y - 2, ir.w + 4, ir.h + 4);
    
    SDL_FillRect(screen, &fr, 255);
	  SDL_BlitSurface(ICON, NULL, screen, &ir);
  }
  SDL_Flip(screen);
}


