// Grafiklib for gp32
// This file is written/copyright 2004 Mirko Roller
// Changed by Chui.

#include <x_gp32.h>

int GP32_NEW_BLU=-1;

void x_gp32_setFramebuffer(unsigned int addr,int vsync)
{
  unsigned int LCDBANK  =  addr >> 22;
  unsigned int LCDBASEU = (addr & 0x3FFFFF) >> 1;
  unsigned int LCDBASEL;
  unsigned short OFFSIZE = 0;
  unsigned short PAGEWIDTH;
  unsigned short vidmode = ((rLCDCON1>>1) & 15)-8;   // result in 0,1,2,3,4
  const unsigned char faktor[]= {15,30,60,120,240};

  LCDBASEL  = LCDBASEU + 320*faktor[vidmode];
  PAGEWIDTH = faktor[vidmode];

  /* (unsigned int*) */ rLCDSADDR1 = (LCDBANK<<21) | (LCDBASEU<<0) ;
  /* (unsigned int*) */ rLCDSADDR2 = (LCDBASEL<<0) ;
  /* (unsigned int*) */ rLCDSADDR3 = (OFFSIZE<<11) | (PAGEWIDTH<<0) ;
  if (vsync)
    x_gp32_waitVsync();
}

short x_gp32_initFramebuffer(unsigned int addr,unsigned short bitmode,unsigned short refreshrate)
{
  unsigned int GPHCLK = x_gp32_GetHCLK();

  if(GP32_NEW_BLU == -1)
  {
    unsigned int biosv=*(int*)0x1014;
    GP32_NEW_BLU = (biosv<0x01030606) ? 0 : 1;
    
    x_gp32_ReadKeys();
    if( x_gp32_nKeys & BUTTONMASK_LEFT) GP32_NEW_BLU = 0;
    if( x_gp32_nKeys & BUTTONMASK_RIGHT) GP32_NEW_BLU = 1;
  }

  {
    unsigned short BPPMODE = 12;     // default 16 Bit
    unsigned short CLKVAL = 3;       // default 16 Bit
    unsigned short ENVID = 1;
    unsigned short MMODE = 0;
    unsigned short PNRMODE = 3;
    switch (bitmode)
    {
        //case 16 : BPPMODE=12; break;
      case  8 :
        BPPMODE=11;
        break;
      case  4 :
        BPPMODE=10;
        break;
      case  2 :
        BPPMODE= 9;
        break;
      case  1 :
        BPPMODE= 8;
        break;
    }
    // Get the correct CLKVAL for refreshrate
    // works in all bitmodes now :)
    if (refreshrate <  50)
      refreshrate = 50;
    if (refreshrate > 120)
      refreshrate =120;

    if(GP32_NEW_BLU)
    {
      CLKVAL = (GPHCLK/(109850*2*refreshrate))-1;
      if (CLKVAL == 0)
        CLKVAL=1;
      refreshrate = GPHCLK / (109850*2*(CLKVAL+1));
    }
    else
    {
      CLKVAL = (GPHCLK/(83385*2*refreshrate))-1;
      if (CLKVAL == 0)
        CLKVAL=1;
      refreshrate = GPHCLK / (83385*2*(CLKVAL+1));
    }

    /* (unsigned int*) */ rLCDCON1 = (CLKVAL<<8) | (MMODE<<7) | (PNRMODE<<5) | (BPPMODE<<1) | (ENVID<<0) ;
  }
  {
    unsigned short LINEVAL = 320-1;

    unsigned short VBPD = GP32_NEW_BLU ? 8 : 1;
    unsigned short VFPD = GP32_NEW_BLU ? 2 : 2;
    unsigned short VSPW = GP32_NEW_BLU ? 5 : 1;
    /* (unsigned int*) */ rLCDCON2 = 0;
    /* (unsigned int*) */ rLCDCON2 = (VBPD<<24) | (LINEVAL<<14) | (VFPD<<6) | (VSPW<<0) ;
  }
  {
    unsigned short HBPD = GP32_NEW_BLU ? 50 : 6;
    unsigned short HFPD = 2;
    unsigned short HOZVAL = 240-1;
    /* (unsigned int*) */ rLCDCON3 = 0;
    /* (unsigned int*) */ rLCDCON3 = (HBPD<<19) | (HOZVAL<<8) | (HFPD<<0) ;
  }
  {
    unsigned short ADDVAL = 0;
    unsigned short HSPW = GP32_NEW_BLU ? 30 : 4;
    unsigned short MVAL = 0;
    unsigned short PALADDEN = 0;
    /* (unsigned int*) */ rLCDCON4 = 0;
    /* (unsigned int*) */ rLCDCON4 = (PALADDEN<<24) | (ADDVAL<<16) | (MVAL<<8) | (HSPW<<0) ;
  }
  {
    unsigned short BSWP = 0;
    unsigned short ENLEND = 0;
    unsigned short HWSWP = 1;
    unsigned short INVENDLINE = 0;
    unsigned short INVVCLK = 1;
    unsigned short INVVD = 0;
    unsigned short INVVDEN = 0;
    unsigned short INVVFRAME = 1;
    unsigned short INVVLINE = 1;
    if (bitmode<16)
    { BSWP=1;HWSWP=0; }
    /* (unsigned int*) */ rLCDCON5 = 0;
    /* (unsigned int*) */ rLCDCON5 = (INVVCLK<<10) | (INVVLINE<<9) | (INVVFRAME<<8) | (INVVD<<7) | (INVVDEN<<6)
                               | (INVENDLINE<<4) | (ENLEND<<2) | (BSWP<<1) | (HWSWP<<0) ;
  }
  x_gp32_setFramebuffer(addr,1);

  return refreshrate;
}


void x_gp32_setPalette( unsigned char pos,unsigned short color)
{
  unsigned int *palette = (unsigned int*) LCD_PALETTE;
  // writing to palette is only allowed, if VSTATUS
  // is not in active mode
  while ((rLCDCON5>>19) == 2)
    ;
  palette[pos]=color;
}

